{% load pages_tags mezzanine_tags i18n future %}
<meta name="keywords" content="{% block meta_keywords %}{% endblock %}">
<meta name="description" content="{% block meta_description %}{% endblock %}">
<title>{% block meta_title %}{% endblock %}{% if settings.SITE_TITLE %} | {{ settings.SITE_TITLE }}{% endif %}</title>
<link rel="shortcut icon" href="{{ STATIC_URL }}img/favicon.ico">

{% ifinstalled mezzanine.blog %}
<link rel="alternate" type="application/rss+xml" title="RSS" href="{% url "blog_post_feed" "rss" %}">
<link rel="alternate" type="application/atom+xml" title="Atom" href="{% url "blog_post_feed" "atom" %}">
{% endifinstalled %}

{% compress css %}
<--<link rel="stylesheet" href="{{ STATIC_URL }}/apps/bootstrap3/css/bootstrap.css">-->

<link rel="stylesheet" href="{{ STATIC_URL }}css/mezzanine.css">
{% ifinstalled cartridge.shop %}
<link rel="stylesheet" href="{{ STATIC_URL }}css/cartridge.css">
{% endifinstalled %}
{% block extra_css %}{% endblock %}
{% endcompress %}

{% compress js %}
<script src="{{ STATIC_URL }}mezzanine/js/{{ settings.JQUERY_FILENAME }}"></script>
<script src="{{ STATIC_URL }}js/bootstrap.js"></script>
<script>
$(function() {
    $('.middleinput:text, textarea').addClass('xlarge');
    $('.control-group label').addClass('control-label');
});
</script>
{% block extra_js %}{% endblock %}
{% endcompress %}


<!--[if lt IE 9]>
<script src="{{ STATIC_URL }}js/html5shiv.js"></script>
<![endif]-->

{% block extra_head %}{% endblock %}
